# Primary application ASG and ELB

data "template_file" "app_user_data" {
  template = "${file("${path.module}/asg_puppetizer.template")}"
  vars {
    hostname_prefix = "${var.hostname}-aws"
    domain = "${lookup(var.domain, var.environment)}"
    cluster = "${var.name}"
    environment = "${var.environment}"
  }
}

resource "aws_launch_configuration" "app_conf" {
  name_prefix = "${var.name}-${var.environment}-app-"
  image_id = "${var.ami}"
  instance_type = "${var.instance_type}"
  security_groups = ["${data.terraform_remote_state.account.twitch_subnets_sg}"]
  user_data = "${data.template_file.app_user_data.rendered}"
  iam_instance_profile = "${aws_iam_instance_profile.app.id}"

  root_block_device {
    volume_size = "${var.root_volume_size}"
  }

  lifecycle {
    create_before_destroy = true
  }
}

resource "aws_autoscaling_group" "app_asg" {
  name = "${var.name}-${var.environment}-app"
  vpc_zone_identifier = ["${data.terraform_remote_state.account.service_a}",
                          "${data.terraform_remote_state.account.service_b}",
                          "${data.terraform_remote_state.account.service_c}"]
  max_size = "${var.max_size}"
  min_size = "${var.min_size}"
  desired_capacity = "${var.desired_capacity}"
  health_check_grace_period = 600
  health_check_type = "EC2"
  load_balancers = ["${aws_elb.internal_app_elb.name}"]
  launch_configuration = "${aws_launch_configuration.app_conf.name}"

  tag {
    key = "Name"
    value = "${var.name}"
    propagate_at_launch = true
  }
  tag {
    key = "Owner"
    value = "${var.owner}"
    propagate_at_launch = true
  }
  tag {
    key = "Project"
    value = "${var.project}"
    propagate_at_launch = true
  }
}

module "asg_dns" {
  source = "git::git+ssh://git@git-aws.internal.justin.tv/web/asg-r53-lambda.git//modules/notifications"
  topic_name = "${var.name}_${var.environment}_topic"
  asg_name = "${aws_autoscaling_group.app_asg.name}"
  lambda_arn = "${data.terraform_remote_state.asg_lambda.asg_lambda_arn}"
}

resource "aws_elb" "internal_app_elb" {
  name = "${var.name}-${var.environment}-app"
  security_groups = ["${data.terraform_remote_state.account.twitch_subnets_sg}"]
  subnets = ["${split(",", "${data.terraform_remote_state.account.service_subnets}")}"]
  internal = true
  cross_zone_load_balancing = true
  connection_draining = true

  listener {
    instance_port = 8000
    instance_protocol = "http"
    lb_port = 80
    lb_protocol = "http"
  }

  health_check {
    healthy_threshold = 3
    unhealthy_threshold = 5
    timeout = 10
    target = "HTTP:8000/debug/running"
    interval = 30
  }

  tags {
    Name = "${var.name}-${var.environment}-app"
    Owner = "${var.owner}"
    Project = "${var.project}"
  }
}