variable "ami" {
}
variable "upstream_master" {
  description = "The source host for data provisioning"
}
variable "pg_cluster" {
  description = "Things like 'prod', 'replay', etc to uniquely identify the postgres cluster of master and replicas"
}
variable "r53_zone_id" {
  description = "Tag used on resource tags and names to help locate resources in AWS"
}
variable "vpc_security_group_ids" {
  description = "Comma separated list of security group ids"
}
variable "subnet_id" {
  description = "Use the subnet to sprinkle hosts among availability zones"
}
variable "twitch_environment" {
  description = "Used as the fact for development, production, and staging"
}

variable "aws_account" {
  description = "twitch account to use"
  default = "twitch-web-aws"
}
variable "cluster" {
  default = "rails-postgres"
  description = "The puppet hiera cluster for configuration"
}
variable "hostname_prefix" {
  default = "rails-postgres"
  description = "The dns name prefix to use -- usually the same as var.cluster"
}
variable "instance_name_prefix"{
  default = "rails-postgres"
}
variable "instance_role" {
  default = "postgres"
  description = "iam role for the node in ec2"
}
variable "instance_type" {
  default = "i2.8xlarge"
}
variable "owner_tag" {
  default = "d8a <d8a@justin.tv>"
  description = "Tag used on resource tags and names to help locate resources in AWS"
}
variable "pg_role" {
  description = "Role of the node in the cluster, eg 'hot-spare'"
  default = "read-replica"
}
variable "project_tag" {
  default = "web/rails-postgres"
  description = "Tag used on resource tags and names to help locate resources in AWS"
}

data "template_file" "master_20170602_ud" {
  template="${file("${path.module}/../templates.20170314/sitedb.user_data")}"
  vars {
    aws_account="${var.aws_account}"
    cluster="${var.cluster}"
    hostname_prefix="${var.instance_name_prefix}"
    pg_cluster="${var.pg_cluster}"
    pg_role="master"
    twitch_environment="${var.twitch_environment}"
    upstream_master="${var.upstream_master}"
  }
}


resource "aws_instance" "instance" {
  ami = "${var.ami}"
  associate_public_ip_address = "false"
  iam_instance_profile = "${var.instance_role}"
  instance_type = "${var.instance_type}"
  subnet_id = "${var.subnet_id}"
  user_data = "${data.template_file.master_20170602_ud.rendered}"
  vpc_security_group_ids = ["${split(",", var.vpc_security_group_ids)}"]

  # These block devices are referenced in the prepare-sitedb script
  ephemeral_block_device {
    device_name = "/dev/xvdf"
    virtual_name = "ephemeral0"
  }
  ephemeral_block_device {
     device_name = "/dev/xvdg"
     virtual_name = "ephemeral1"
  }
  ephemeral_block_device {
    device_name = "/dev/xvdh"
    virtual_name = "ephemeral2"
  }
  ephemeral_block_device {
    device_name = "/dev/xvdi"
    virtual_name = "ephemeral3"
  }
  ephemeral_block_device {
    device_name = "/dev/xvdj"
    virtual_name = "ephemeral4"
  }
  ephemeral_block_device {
    device_name = "/dev/xvdk"
    virtual_name = "ephemeral5"
  }
  ephemeral_block_device {
    device_name = "/dev/xvdl"
    virtual_name = "ephemeral6"
  }
  ephemeral_block_device {
    device_name = "/dev/xvdm"
    virtual_name = "ephemeral7"
  }
  root_block_device {
    volume_size = "500"
    volume_type = "gp2"
  }
  tags {
    Name = "${var.cluster}/${var.pg_cluster}/${var.pg_role}"
    Owner = "${var.owner_tag}"
    Project = "${var.project_tag}"
  }
}

resource "aws_route53_record" "dns" {
  name = "${var.hostname_prefix}-${replace(aws_instance.instance.id, "i-", "")}.${var.twitch_environment}.${var.aws_account}.us-west2.justin.tv"
  provider = "aws.dns"
  records = ["${aws_instance.instance.private_ip}"]
  ttl = "300"
  type = "A"
  zone_id = "${var.r53_zone_id}"
}
